/***************************************************************************
 *   Copyright (C) 2005 by Pawel Nawrocki                                  *
 *   pnawrocki@interia.pl                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "wlassistant.h"
#include "waconfig.h"
#include "ui_confpaths.h"
#include "ui_confgeneral.h"
#include "ui_confwep.h"
#include "ui_wepselect.h"
#include "ui_confiface.h"
#include "ui_confadvanced.h"
#include "netlistviewitem.h"
//#include <stdlib.h>
//#include <unistd.h>

#include <qlabel.h>
#include <qlistview.h>
#include <qprocess.h>
#include <qcursor.h>
#include <qlistbox.h>
#include <qprogressbar.h>
#include <qpainter.h>
#include <qregexp.h>
#include <qtooltip.h>

//#include <qapplication.h>
#include <qeventloop.h>
#include <qtimer.h>

#include <kcombobox.h>
#include <klistview.h>
#include <kapplication.h>
#include <kconfigdialog.h>
#include <kiconloader.h>
#include <kurlrequester.h>
#include <kmessagebox.h>
#include <kstatusbar.h>

WirelessAssistant::WirelessAssistant(QWidget* parent, const char* name, bool modal, WFlags fl)
    : mainWindow(parent,name, modal,fl)
{
buttonScan->setIconSet( SmallIcon("find") );
buttonConnect->setIconSet( SmallIcon("connect_creating") );
buttonSettings->setIconSet( SmallIcon("configure") );
buttonClose->setIconSet( SmallIcon("fileclose") );
buttonStop->setIconSet( SmallIcon("stop") );

netList->setAllColumnsShowFocus(1);
netList->setTooltipColumn(0);

netList->setColumnAlignment(1, Qt::AlignRight );
netList->setColumnAlignment(2, Qt::AlignRight );
netList->setAlternateBackground( QColor(238, 246, 255) );
netList->setSorting( 0, 1 );


  connect(buttonDetect, SIGNAL(clicked()),
          this, SLOT(setDeviceList()) );

startup=1;
updateConfiguration();
QTimer::singleShot(10, this, SLOT(setDeviceList()) ); //WAIT FOR THE UI TO BE READY BEFORE DEVICE DETECTION (msec)
}

WirelessAssistant::~WirelessAssistant()
{}

/*$SPECIALIZATION$*/

void WirelessAssistant::setDeviceList()
{
  QString result;
  statusLabel->setText("Detecting wireless devices...");
  statusLabel->repaint();
  setUi(0);
  if (devCombo->count() > 0) devCombo->clear();
  
  result = runCommand( WaSettings::pathIw() );
  if ( result == "TERM" ) return;
  else if ( result != "ERR" ) parseDevDetect( result );
}

void WirelessAssistant::parseDevDetect( QString output )
{
  QString devItem;
  QString iface;
  int mark1;
  output.simplifyWhiteSpace();
  mark1 = output.find( QRegExp("[a-zA-Z]{3,4}\\d ")); 
  while ( mark1 > -1) {
  	iface = output.mid(mark1, output.find(" ", mark1));
  	devItem = getIfName(iface);
  	if (!devItem.isNull()) devItem = QString(iface+" ("+devItem+")");
  	else devItem=iface;
  	devCombo->insertItem( devItem, -1 );
  	mark1 = output.find( QRegExp("[a-zA-Z]{3,4}\\d "), mark1+2); 
  }

  uiDevDetected();
}

void WirelessAssistant::uiDevDetected()
{
  if (devCombo->count() > 0)
  {
    devCombo->setCurrentItem( 0 );
    statusLabel->setText("Device detection completed.");
    if ( (WaSettings::autoScan()) && (startup)) netScan();
  }
  else
    statusLabel->setText("No wireless devices found!");
  startup=0;
  setUi(1);
}

void WirelessAssistant::netConnect()
{
  if (!netList->selectedItem()) { if ( !WaSettings::singleClick() ) showErr(4); return;}
  
  QString wepkey = QString::null;
  QString wepmode = QString::null;
  QString cmd,result;
  
  if (netList->selectedItem()->pixmap(4)) {	//is selected network encrypted?
	statusLabel->setText("Waiting for a WEP key...");
	QString wepstring  = getWepKey();
	if (wepstring.isEmpty()) return; // wep selection cancelled - exit function.
	else {
		wepkey = wepstring.section(",",0,0);
		wepmode = wepstring.section(",",1,1);
	}
  }
  
  statusLabel->setText( QString("Connecting to '")+netList->selectedItem()->text(0)+("'...") );
  setUi(0);
  
  cmd = QString(
  	WaSettings::pathIw() + " " +
  	getDev() + 
  	" essid " +
  	"'" + netList->selectedItem()->text(0) + "' mode " +
  	netList->selectedItem()->text(2) +
  	" channel " +
  	netList->selectedItem()->text(1) +
	" key ");
	
  if (wepkey.isEmpty()) cmd.append("off"); //no encryption?
  else {	//encryption enabled
  	cmd.append(wepmode);
  	cmd.append(" ");
  	cmd.append(wepkey);
  }
  result = runCommand( cmd );
  if ( result == "TERM" || result == "ERR" ) return;

    
  if (WaSettings::ifaceManual()) { // MANUAL INTERFACE CONFIG using 'ifconfig'
  	clearDhcp(); //should this be run?
  		
  	cmd = QString(
  	WaSettings::pathIf()+ " " +
  	getDev()+ " " +
  	WaSettings::ifaceIP() );
  	
  	QString ifaceStr;
  	ifaceStr = WaSettings::ifaceNetmask();
  	if (!ifaceStr.isEmpty()) {
  		cmd.append(" netmask ");
  		cmd.append( ifaceStr );
  	}
  	
  	ifaceStr = WaSettings::ifaceBroadcast();
  	if (!ifaceStr.isEmpty()) {
  		cmd.append(" broadcast ");
  		cmd.append( ifaceStr );
  	}
  	result = runCommand( cmd );
  }
  else if ( netList->selectedItem()->text(2)!="Ad-Hoc" ) {	//AUTOMATIC IFACE CONFIG (and NOT ad-hoc)
  	clearDhcp();
  	QString dhcpClient = WaSettings::pathDhcp().section("/",-1,-1);
  	if (dhcpClient == "dhcpcd") {
  		cmd = QString(
  		WaSettings::pathDhcp() +
  		" -nd -t " +
  		QString::number(WaSettings::dhcpTimeout()) + " " +
  		getDev() );
  	}
  	else {	//dhclient
  		cmd = QString( WaSettings::pathDhcp()+" "+getDev() );
  		if (WaSettings::authorizeDhclient()) {
  			int authMethod = WaSettings::dhclientAuthorization().toInt();
  			if (authMethod == 0) cmd.prepend( "kdesu " );
  			else if (authMethod == 1) cmd.prepend( "sudo ");
  		}
  	}
  	
  	statusLabel->setText("Getting IP information...");
  	
  	result = runCommand( cmd );
  }

  bool ok = 0;
  if ( result == "TERM" ) return;
  else if ( result != "ERR" ) {
  	ok = connectionOk();
  	if (ok && (WaSettings::runCustom()) ) runCommand( WaSettings::customCommand(), 2 );	
  }
  uiConnected(ok);
}

void WirelessAssistant::uiConnected( bool ok )
{
  if ( (ok) && ( netList->selectedItem()->text(2)!="Ad-Hoc" ) ) ok = connectionOk();
  if (ok) {
  	if (WaSettings::autoQuit()) this->close();
  	if ( WaSettings::ifaceAuto() ) statusLabel->setText( QString("Connection with '")+netList->selectedItem()->text(0) +("' successfully established.") );
  	else statusLabel->setText( QString("Network parameters for '")+netList->selectedItem()->text(0) +("' successfully set.") );
  }
  else statusLabel->setText("Connecting failed!");
  
  setUi(1);
}

void WirelessAssistant::netScan()
{  
  if (devCombo->count() == 0) return;
  
  QString result;
  
  /*if (getDev().find("ath")>-1) masterToManaged=1; //MADWIFI DRIVERS ERROR WORKAROUND (they sometimes report Managed networks as Master).
  	else masterToManaged=0;*/	//now default for all cards.
  setUi(0);
  netList->clear();
  statusLabel->setText("Bringing interface up...");

  result = runCommand( QString(
  WaSettings::pathIf() + " " +
  getDev() + " " +
  "up" ) );
  
  if ( result == "ERR" || result == "TERM" ) return;

  // *** CHANGE MODE TO MANAGED BEFORE SCANNING. Otherwise returns _VERY_ limited results.
  runCommand( QString( 
  	WaSettings::pathIw() + " " +
  	getDev() + " " +
  	"mode Managed") );
  
  statusLabel->setText("Scanning...");

  result = runCommand( QString(
  		WaSettings::pathIwl() + " " +
  		getDev() + " " +
  		"scanning") );

  if ( result == "ERR" || result == "TERM" ) return;
  else parseScan( result );
}

void WirelessAssistant::parseScan( QString output )
{
QString essid;
QString channel;
QString mode;
//QString quality;
int qualInt;
bool enc;

bool ok_channel = 1;
bool ok_noise = 1;//does iwlist return channel/noise?

  for (int i=0; (!output.section("Cell ",i,i).isEmpty()); i++ )
  {
    // GET ESSID VALUE
    essid = getVal(output, "ESSID:\"","\"","Cell ",i);
	if (essid.isEmpty()) continue;
	
    // GET CHANNEL NUMBER
    channel = getVal(output, "Channel:"," ","Cell ",i);
    if (channel.isEmpty()) {
    	channel = getVal(output, "Frequency:","Hz","Cell ",i);
    	ok_channel = 0;
    }
    
    // GET MODE VALUE
    mode = getVal(output, "Mode:"," ","Cell ",i);
	//if ( (mode.upper()=="AD-HOC") && 
	if (mode.upper()=="MASTER") mode="Managed"; //MADWIFI DRIVERS ERROR WORKAROUND (they report Managed networks as Master).
	if ( (mode.upper()=="AD-HOC") && (WaSettings::skipAdHoc()) ) continue;
	
    // GET QUALITY
    	int wnoise, wsignal;
    	//check if driver reports quality directly
    	qualInt = getVal(output, "Quality","/","Cell ",i,1).toInt();

    	if (qualInt == 0) { //not directly reported? What a pity.
    		wnoise = getVal(output, "Noise level","dBm","Cell ",i,2).toInt();
    		wsignal = getVal(output, "Signal level","dBm","Cell ",i,2).toInt();
    		if (wnoise!=0) qualInt = int( (wnoise-wsignal)*100 /wnoise ); //noise reported? cool.
    			else {		//noise not detected? do some estimation.
    			qualInt = 100-wsignal;
    			if (qualInt <1) qualInt=1;
			ok_noise = 0;
    		}
    	}
    if (qualInt>94) qualInt=94;

      
    // GET ENCRYPTION
    if (getVal(output, "Encryption key:"," ","Cell ",i).upper()=="ON") {
    	if (WaSettings::skipEnc()) continue; //skip encrypted networks
    	enc = 1;
    }
    else
    	enc = 0;

    new NetListViewItem( netList, essid, channel, mode, qualInt, enc );
  }
  
  if (!ok_channel) netList->setColumnText( 1, "Freq (Hz)" );
  if (!ok_noise) netList->setColumnText( 3, "Signal level" );
  uiScanned();
}

void WirelessAssistant::uiScanned()
{
  setNetListColumns();
  
  if (netList->childCount() > 0)
  {
    buttonScan->setText("Re&scan");
    QString msg = "";
    if (WaSettings::skipEnc() && WaSettings::skipAdHoc()) msg = "Skipped ad-hoc & encrypted networks.";
    if (WaSettings::skipEnc() && !WaSettings::skipAdHoc()) msg = "Skipped encrypted networks.";
    if (!WaSettings::skipEnc() && WaSettings::skipAdHoc()) msg = "Skipped ad-hoc networks.";
    if (WaSettings::singleClick()) msg.append(" Click on an item to connect.");
    statusLabel->setText( QString("Scanning completed. "+msg));
  }
  else
  {
    statusLabel->setText("No wireless networks found!");
  }
  setUi(1);
}

void WirelessAssistant::showCfgDlg()
{

if ( KConfigDialog::showDialog( "conf" ) ) return; 
 
KConfigDialog* dialog = new KConfigDialog( this, "conf", WaSettings::self() );

ui_confGeneral* confG = new ui_confGeneral();
dialog->addPage( confG, "General", "kcontrol" );

ui_confWep* confW = new ui_confWep();
dialog->addPage( confW, "WEP Keys", "password" );

ui_confIface* confI = new ui_confIface();
dialog->addPage( confI, "Interface", "hwinfo" );

ui_confPaths* confP = new ui_confPaths();
dialog->addPage( confP, "Paths", "kfm");
confP->buttonDetect->setIconSet( SmallIcon("misc") );

ui_confAdvanced* confA = new ui_confAdvanced();
dialog->addPage( confA, "Advanced", "misc");

connect( confP->buttonDetect, SIGNAL(clicked()),
	 confP, SLOT(detectDevices()) );

connect( dialog, SIGNAL(settingsChanged()), 
         this, SLOT(updateConfiguration()) ); 
         
dialog->show();
}

void WirelessAssistant::updateConfiguration()
{
	if (WaSettings::singleClick()) {
		setSingleClick( 1 );
		buttonConnect->hide();
	}
	else {
		setSingleClick( 0 );
		buttonConnect->show();
	}
}

void WirelessAssistant::setSingleClick(bool i)
{
	if (i) connect( netList, SIGNAL(clicked(QListViewItem*)),
			this, SLOT(netConnect()) );
	else netList->disconnect();
}

QString WirelessAssistant::getDev()
{
QString dvn;
int marker;
	dvn = devCombo->currentText();
	marker=dvn.find(" ");
	if (marker>-1) dvn=dvn.left(marker);

	return dvn;
}

QString WirelessAssistant::getIfName(QString ifName)
{
QString devId,devName,execpath;
int marker,len;

	// get device id
	execpath=getPath("bin/udevinfo",1);
	if (execpath.isEmpty()) return QString::null;
	QProcess* proc = new QProcess( this );
	proc->addArgument(execpath);
	proc->addArgument("-ap");
	proc->addArgument( QString("/class/net/"+ifName) );
	if (!proc->start()) return QString::null;
		while (proc->isRunning()) {}
	devId = QString(proc->readStdout());
	marker=devId.find("\"device\"");
	marker=devId.find("ID=",marker);
	marker=devId.find(":",marker)+1;
	len = devId.find("\"",marker)-marker;
	devId=devId.mid(marker, len);

	//get human-readable interface name
	execpath=getPath("bin/lspci",1);
	if (execpath.isEmpty()) return QString::null;
	proc = new QProcess( this );
	proc->addArgument(execpath);
	if (!proc->start()) return QString::null;
		while (proc->isRunning()) {}
	devName = QString(proc->readStdout());
	marker=devName.find(devId)+devId.length();
	marker=devName.find(":",marker)+2;
	len=devName.find("(",marker)-1-marker;
	devName=devName.mid(marker,len);
	
	delete proc;
	return devName;
}

QString WirelessAssistant::getWepKey()      // returns "KEY,MODE"
{
QString key = QString::null;
QString mode = "open";

	if (WaSettings::autoWep()) {	// TRY TO MATCH A PREDEFINED KEY IF ENABLED
		int i = 1;
		QString keyname = QString::null;
		QString essid = netList->selectedItem()->text(0); //get ESSID to match
		
		if (!WaSettings::autoWepCase()) essid = essid.upper();
		while (!WirelessAssistant::getKey(i).isEmpty()) { //check ALL predefined entries. getKey returns null when i out of range
			keyname = WirelessAssistant::getKey(i, 1);
			if (!WaSettings::autoWepCase()) keyname = keyname.upper();
			if (keyname == essid) key = WirelessAssistant::getKey(i);
			i++;
		}

		if (!key.isEmpty()) {	//matching key found
			if (!WaSettings::defaultOpen()) mode = "restricted";	//get default wep mode;
		}
	}
	
	if (key.isEmpty()) {	// NO MATCHING KEY FOUND OR KEY MATCHING DISABLED. SHOW DIALOG.
		
		ui_wepSelect* dlgWepSel = new ui_wepSelect( 0 , "wepselect", 1 );
		dlgWepSel->buttonOk->setIconSet( SmallIcon("ok") );
		dlgWepSel->buttonCancel->setIconSet( SmallIcon("cancel") );

		if ( dlgWepSel->exec() == QDialog::Accepted ) {
			key = dlgWepSel->getWKey();
			mode = dlgWepSel->getWepMode();
		}
		else { 
			statusLabel->setText("Cancelled.");
		}
		delete dlgWepSel;
	}
	if (!key.isEmpty()) key = QString( key + "," + mode );
	return key;
}


////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////

QString WirelessAssistant::runCommand ( QString cmd, int errLevel )
{
	QString perr;
	QString result = "UNKNOWN";
	
	if ( cmd.isEmpty() ) {
		setUi(1);
		KMessageBox::error(0, "Empty path to executable!");
		return "ERR";
	}
	
	///////// DON'T SHOW STATUS IF ERRORS SHALL BE IGNORED
	if (errLevel == 2) {
		statusLabel->setText( QString("Running '"+cmd+"'...") );
		statusLabel->repaint();
	}
	QProcess* p = new QProcess( this );
	int i = 0;
	
	////////// SET PROC ARGUMENTS
	QString arg;
	while ( !cmd.section(" ",i,i).isEmpty() )
	{
		arg = cmd.section(" ",i,i);
		if (arg.left(1) == "'") {	//add strings in '', even with spaces, as single argument
			int j = i;
			while ( cmd.section(" ",i,j).right(1) != "'" ) 	{ j++; }
			arg = cmd.section(" ",i,j);
			arg = arg.mid( 1, arg.length() - 2);	//strip leading and ending '
			i = j;
		}
		p->addArgument( arg );
		i++;
	}
	
	buttonStop->show();
	
	
	if ( !p->start() ) result = "NOT FOUND";

	connect( buttonStop, SIGNAL(clicked()),
		p, SLOT(kill()) );
		
	while ( p->isRunning() ) 
	{ // PROCESS USER EVENTS
		KApplication::eventLoop()->processEvents( QEventLoop::AllEvents );
	}
	
	buttonStop->hide();
	buttonStop->disconnect();
	
	if ( !p->normalExit() && result!="NOT FOUND" ) {
		setUi(1);
		statusLabel->setText("Process terminated.");
		result = "TERM";
	}
		
	if (errLevel>0 && result=="UNKNOWN")
	{
		perr = QString( p->readStderr() ); //CHECK FOR ERRORS
		if ( !perr.isEmpty() ) {
			if ( (perr.upper().find("NOT PERMITTED") > -1) ||
			(perr.upper().find("SUPERUSER") > -1) ||			// TRY TO CATCH COMMON "PERMISSION DENIED" ERRORS
			(perr.upper().find("PERMISSION DENIED") > -1 ) )
			{
				result = "DENIED";
			}
			else if (errLevel==2) result = "ERR";
		}
	}
	
	// NO ERRORS FOUND OR noErrors FLAG ON
	if ( result == "UNKNOWN" ) {
		result = QString( p->readStdout() );
		//if ( result.isEmpty() ) result = "OK";
	}
	else if ( result!="TERM" ) {
		if ( result == "NOT FOUND" ) showErr(1);
  		if ( result == "DENIED" ) {
  			if ( cmd.find("dhclient")>-1 ) KMessageBox::error(0, "You do not have sufficient permissions to run dhclient.\nGo to Advanced configuration and select authorization method that you would like to use.");
  			else showErr(2);
  		}
  		if ( result == "ERR" ) { setUi(1); KMessageBox::error(0, QString("Command '"+cmd+"' returned the following error:\n"+perr) ); }
  		result = "ERR";
  	}
	
	return result;
}

void WirelessAssistant::setUi(int uiState) {
	
		if (uiState==0) { 
			devCombo->setEnabled( false );
    			buttonScan->setEnabled( false );
			buttonConnect->setEnabled( false );
			buttonSettings->setEnabled( false );
			buttonDetect->setEnabled( false );
			setSingleClick(0);
			KApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
		}
		else { 
			if (devCombo->count() > 0) { devCombo->setEnabled( true ); buttonScan->setEnabled( true );}
    			if (netList->childCount() > 0) buttonConnect->setEnabled( true );
    			buttonSettings->setEnabled( true );
    			buttonDetect->setEnabled( true );
			if (WaSettings::singleClick()) setSingleClick(1);
			KApplication::restoreOverrideCursor();
		}
}

void WirelessAssistant::setNetListColumns()
{
  int realWidth = netList->viewportSize( netList->contentsWidth(), netList->contentsHeight() ).width(); //calculate actual width taking scrollbars into account
  int essidWidth = realWidth - netList->columnWidth(1) - netList->columnWidth(2) - netList->columnWidth(3) - netList->columnWidth(4);
  
  netList->setColumnWidth(0, essidWidth);
}

void WirelessAssistant::clearDhcp()
{
	statusLabel->setText("Checking if DHCP client is already running...");
	statusLabel->repaint();
	
	QString result;
	QString dhcpClient = WaSettings::pathDhcp().section("/",-1,-1);
	result = runCommand( QString("/sbin/pidof "+dhcpClient) );
	if ( !result.isEmpty() )
	{
		result = runCommand( QString("find /var/run /etc/dhcpc /var/state/dhcp -name "+dhcpClient+"-"+getDev()+".pid"), 0 );
		if ( !result.isEmpty() )
		{
			if (dhcpClient=="dhcpcd") {
				statusLabel->setText("Waiting for DHCP client to shut down...");
				statusLabel->repaint();
			
				runCommand( QString(
					WaSettings::pathDhcp() + " -k " +
					getDev()) );
				QTimer* tmr = new QTimer();
				tmr->start(1500, 1);
				while ( tmr->isActive() ) { KApplication::eventLoop()->processEvents( QEventLoop::AllEvents ); } //wait 0.5sec for dhcpcd to be killed.
			}
			else {	//dhclient
				statusLabel->setText("Waiting for DHCP release current lease...");
				statusLabel->repaint();
			
				runCommand( QString(
					WaSettings::pathDhcp() + " -r " +
					getDev()) );
				QTimer* tmr = new QTimer();
				tmr->start(1500, 1);
				while ( tmr->isActive() ) { KApplication::eventLoop()->processEvents( QEventLoop::AllEvents ); } //wait 0.5sec for dhclient to release.
			}
		}
	}
}

void WirelessAssistant::setSuid() {
  
  QString cmd = QString(
  "kdesu chmod u+s " +
  WaSettings::pathDhcp() + " " +
  WaSettings::pathIf() + " " +
  WaSettings::pathIw() + " " +
  WaSettings::pathIwl());
  if ( WaSettings::pathDhcp().section("/",-1,-1) == "dhclient") {
  	cmd.append( QString(" "+WaSettings::pathDhcp()+"-script") );
  	KMessageBox::error(0, "You are using dhclient. Even after setting proper file permissions, you will need root password while connecting in order to use this client.");
  }
  runCommand( cmd );
}

bool WirelessAssistant::connectionOk() {
	QString res;
	statusLabel->setText("Testing connection..."); statusLabel->repaint();
	res = runCommand( QString("ping -c 1 -I "+getDev()+" www.google.com") );
	if (res.find("Host Unreachable") > -1) res = "ERR";
	if ( res == "ERR" || res == "TERM" ) return 0;
	else return 1;
}

void WirelessAssistant::showErr(int errNum) {

		// executable not found
		setUi(1);
		if (errNum==1) { 
			KMessageBox::error(0, "One of the executables could not be found!\nReview your configuration and make sure that paths to executables are set properly.");
			statusLabel->setText("Exacutable not found!");
			showCfgDlg();
		}
		//permission denied
		else if (errNum==2) {
			if ( KMessageBox::warningYesNo(0,"You have insufficient permissions to perform this action.\nWould you like to set all necessary executables setuid root, so regular users can use them?\nNOTE: If you answer yes, you will need to enter root password.","Insufficient permissions") == 3) setSuid(); //3 = Yes
			statusLabel->setText("Insufficient permissions!");
		}
		else if (errNum==3) {
			KMessageBox::error(0, "Executable 'iwconfig' could not be found!\nReview your configuration and make sure that the paths to executables are set properly.\nPress the 'Detect' button when done to retry.");
			statusLabel->setText("Exacutable not found!");
			showCfgDlg();
		}
		else if (errNum==4) {
			KMessageBox::error(0, "No network selected!");
			statusLabel->setText("Select network and push the 'Connect' button!");
		}
		else {
			KMessageBox::error(0, "Unknown error occured!");
			statusLabel->setText("Unknown error!");
		}
}

QString WirelessAssistant::getPath(QString cmd, bool skipErr)
{
  QString tmpstr;
  QProcess *locateProc;
  
  locateProc = new QProcess();
  
  locateProc->clearArguments();
  locateProc->addArgument("locate");
  locateProc->addArgument(cmd);
  /*for (int i=0; cmd.section(" ",i,i) != ""; i++)
  {
    proc2->addArgument( cmd.section(" ",i,i) );
  }*/
  
  if ( !locateProc->start()) {
  	if (!skipErr) KMessageBox::error(0,"The 'locate' executable is either not installed or not in your PATH.\nPlease fix this in order to use executable path detection."); 
	return QString::null; 
  }
  
  while (locateProc->isRunning()) {  }
  if ( QString(locateProc->readStderr()).find("ermission denied") > -1) {
  	KMessageBox::error(0, "You do not have permissions to access locate database. Path detection will not work.");
  	return "DENIED";
  }
  tmpstr = QString(locateProc->readStdout() ).simplifyWhiteSpace();
  delete locateProc;
  while (tmpstr.left(4) != "/usr" && tmpstr.left(5) != "/sbin" && tmpstr.left(4) != "/bin") {
  	tmpstr = tmpstr.right( tmpstr.length() - tmpstr.find(" ") - 1 );
  	if (tmpstr.length() < 5) return QString::null;
  }
  tmpstr.truncate( tmpstr.find(" ") );
  return tmpstr;  
}

QString WirelessAssistant::getVal(QString str, QString start, QString end, QString sectDelimit, int sectNo, int offset)
{
int mark1,mark2;

if (!sectDelimit.isEmpty()) str = str.section(sectDelimit, sectNo, sectNo);
if (str.isEmpty()) return QString::null;

str.simplifyWhiteSpace();

if (start.isEmpty()) mark1=0;
else { mark1=str.find(start);
       if (mark1==-1) return QString::null;
       mark1+= start.length();
}


if (end.isEmpty()) mark2=str.length();
else mark2=str.find(end,mark1);

if (mark2==-1) return QString::null;

mark1 += offset; //add offset to the beginning
return str.mid(mark1,mark2-mark1).stripWhiteSpace();
}


#include "wlassistant.moc"
